@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2005 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,23 +16,48 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#ifndef COMMON_H_INCLUDED
-#define COMMON_H_INCLUDED
+#ifndef SNDFILE_COMMON_H
+#define SNDFILE_COMMON_H
 
-#include "config.h"
+#include "sfconfig.h"
+
+#include <stdlib.h>
+
+#if HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
 #ifndef SNDFILE_H
-#include <sndfile.h>
+#include "sndfile.h"
+#endif
+
+#ifdef __cplusplus
+#error "This code is not designed to be compiled with a C++ compiler."
+#endif
+
+#ifdef UNUSED
+#elif defined (__GNUC__)
+#	define UNUSED(x) UNUSED_ ## x __attribute__ ((unused))
+#elif defined (__LCLINT__)
+#	define UNUSED(x) /*@unused@*/ x
+#else
+#	define UNUSED(x) x
+#endif
+
+#ifdef __GNUC__
+#	define WARN_UNUSED	__attribute__ ((warn_unused_result))
+#else
+#	define WARN_UNUSED
 #endif
 
 #define	SF_BUFFER_LEN			(8192*2)
-#define	SF_FILENAME_LEN			(256)
-#define	SF_HEADER_LEN			(4096)
+#define	SF_FILENAME_LEN			(512)
+#define	SF_HEADER_LEN			(4100)
 #define	SF_TEXT_LEN				(1024)
 #define SF_SYSERR_LEN			(256)
 #define SF_MAX_STRINGS			(16)
 
-#define	SF_SEEK_ERROR			((sf_count_t) -1)
+#define	PSF_SEEK_ERROR			((sf_count_t) -1)
 
 
 #define	BITWIDTH2BYTES(x)	(((x) + 7) / 8)
@@ -88,12 +113,10 @@ enum
 	SF_FORMAT_OGG			= 0x4090000,
 
 	SF_FORMAT_REX			= 0x40A0000,		/* Propellorheads Rex/Rcy */
-	SF_FORMAT_SD2			= 0x40C0000,		/* Sound Designer 2 */
 	SF_FORMAT_REX2			= 0x40D0000,		/* Propellorheads Rex2 */
 	SF_FORMAT_KRZ			= 0x40E0000,		/* Kurzweil sampler file */
 	SF_FORMAT_WMA			= 0x4100000,		/* Windows Media Audio. */
 	SF_FORMAT_SHN			= 0x4110000,		/* Shorten. */
-	SF_FORMAT_FLAC			= 0x4120000,
 
 	/* Unsupported encodings. */
 	SF_FORMAT_VORBIS		= 0x1001,
@@ -105,24 +128,45 @@ enum
 } ;
 
 /*---------------------------------------------------------------------------------------
-**	PEAK_CHUNK_OLD - This chunk type is common to both AIFF and WAVE files although their
+**	PEAK_CHUNK - This chunk type is common to both AIFF and WAVE files although their
 **	endian encodings are different.
 */
 
 typedef struct
-{	float			value ;		/* signed value of peak */
-	unsigned int	position ;	/* the sample frame for the peak */
+{	double		value ;		/* signed value of peak */
+	sf_count_t	position ;	/* the sample frame for the peak */
 } PEAK_POS ;
 
 typedef struct
-{	unsigned int	version ;	/* version of the PEAK chunk */
+{	/* libsndfile internal : write a PEAK chunk at the start or end of the file? */
+	int				peak_loc ;
+
+	/* WAV/AIFF */
+	unsigned int	version ;	/* version of the PEAK chunk */
 	unsigned int	timestamp ;	/* secs since 1/1/1970  */
+
+	/* CAF */
+	unsigned int	edit_number ;
+
 #if HAVE_FLEXIBLE_ARRAY
-	PEAK_POS		peaks [] ;	/* the per channel peak info */
+	/* the per channel peak info */
+	PEAK_POS		peaks [] ;
 #else
-	PEAK_POS		peaks [1] ;	/* the per channel peak info */
+	/*
+	** This is not ISO compliant C. It works on some compilers which
+	** don't support the ISO standard flexible struct array which is
+	** used above. If your compiler doesn't ike this I suggest you find
+	** youself a 1999 ISO C standards compilant compiler. GCC-3.X is
+	** highly recommended.
+	*/
+	PEAK_POS		peaks [0] ;
 #endif
-} PEAK_CHUNK ;
+} PEAK_INFO ;
+
+static inline PEAK_INFO *
+peak_info_calloc (int channels)
+{	return calloc (1, sizeof (PEAK_INFO) + channels * sizeof (PEAK_POS)) ;
+} /* peak_info_calloc */
 
 typedef struct
 {	int		type ;
@@ -138,8 +182,25 @@ typedef struct
 
 typedef struct sf_private_tag
 {	/* Force the compiler to double align the start of buffer. */
-	double			buffer		[SF_BUFFER_LEN / sizeof (double)] ;
-	char			filename	[SF_FILENAME_LEN] ;
+	union
+	{	double			dbuf	[SF_BUFFER_LEN / sizeof (double)] ;
+#if (defined (SIZEOF_INT64_T) && (SIZEOF_INT64_T == 8))
+		int64_t			lbuf	[SF_BUFFER_LEN / sizeof (int64_t)] ;
+#else
+		long			lbuf	[SF_BUFFER_LEN / sizeof (double)] ;
+#endif
+		float			fbuf	[SF_BUFFER_LEN / sizeof (float)] ;
+		int				ibuf	[SF_BUFFER_LEN / sizeof (int)] ;
+		short			sbuf	[SF_BUFFER_LEN / sizeof (short)] ;
+		char			cbuf	[SF_BUFFER_LEN / sizeof (char)] ;
+		signed char		scbuf	[SF_BUFFER_LEN / sizeof (signed char)] ;
+		unsigned char	ucbuf	[SF_BUFFER_LEN / sizeof (signed char)] ;
+		} u ;
+
+	char			filepath	[SF_FILENAME_LEN] ;
+	char			rsrcpath	[SF_FILENAME_LEN] ;
+	char			directory	[SF_FILENAME_LEN] ;
+	char			filename	[SF_FILENAME_LEN / 4] ;
 
 	char			syserr		[SF_SYSERR_LEN] ;
 
@@ -167,15 +228,30 @@ typedef struct sf_private_tag
 	int				has_text ;
 	int				do_not_close_descriptor ;
 
-	int 			filedes ;
+#if USE_WINDOWS_API
+	/*
+	**	These fields can only be used in src/file_io.c.
+	**	They are basically the same as a windows file HANDLE.
+	*/
+	void 			*hfile, *hrsrc, *hsaved ;
+#else
+	/* These fields can only be used in src/file_io.c. */
+	int 			filedes, rsrcdes, savedes ;
+#endif
 
-	int				end_of_file ;
 	int				error ;
 
 	int				mode ;			/* Open mode : SFM_READ, SFM_WRITE or SFM_RDWR. */
 	int				endian ;		/* File endianness : SF_ENDIAN_LITTLE or SF_ENDIAN_BIG. */
 	int				float_endswap ;	/* Need to endswap float32s? */
 
+	/*
+	** Maximum float value for calculating the multiplier for
+	** float/double to short/int conversions.
+	*/
+	int				float_int_mult ;
+	float			float_max ;
+
 	/* Vairables for handling pipes. */
 	int				is_pipe ;		/* True if file is a pipe. */
 	sf_count_t		pipeoffset ;	/* Number of bytes read from a pipe. */
@@ -186,13 +262,16 @@ typedef struct sf_private_tag
 	SF_INFO			sf ;
 
 	int				have_written ;	/* Has a single write been done to the file? */
-	int				has_peak ;		/* Has a PEAK chunk (AIFF and WAVE) been read? */
-	int				peak_loc ;		/* Write a PEAK chunk at the start or end of the file? */
-	PEAK_CHUNK		*pchunk ;
+	PEAK_INFO		*peak_info ;
+
+	/* Loop Info */
+	SF_LOOP_INFO	*loop_info ;
 
 	sf_count_t		filelength ;	/* Overall length of (embedded) file. */
 	sf_count_t		fileoffset ;	/* Offset in number of bytes from beginning of file. */
 
+	sf_count_t		rsrclength ;	/* Length of the resource fork (if it exists). */
+
 	sf_count_t		dataoffset ;	/* Offset in number of bytes from beginning of file. */
 	sf_count_t		datalength ;	/* Length in bytes of the audio data. */
 	sf_count_t		dataend ;		/* Offset to file tailer. */
@@ -227,10 +306,10 @@ typedef struct sf_private_tag
 	sf_count_t		(*read_float)	(struct sf_private_tag*, float *ptr, sf_count_t len) ;
 	sf_count_t		(*read_double)	(struct sf_private_tag*, double *ptr, sf_count_t len) ;
 
-	sf_count_t		(*write_short)	(struct sf_private_tag*, short *ptr, sf_count_t len) ;
-	sf_count_t		(*write_int)	(struct sf_private_tag*, int *ptr, sf_count_t len) ;
-	sf_count_t		(*write_float)	(struct sf_private_tag*, float *ptr, sf_count_t len) ;
-	sf_count_t		(*write_double)	(struct sf_private_tag*, double *ptr, sf_count_t len) ;
+	sf_count_t		(*write_short)	(struct sf_private_tag*, const short *ptr, sf_count_t len) ;
+	sf_count_t		(*write_int)	(struct sf_private_tag*, const int *ptr, sf_count_t len) ;
+	sf_count_t		(*write_float)	(struct sf_private_tag*, const float *ptr, sf_count_t len) ;
+	sf_count_t		(*write_double)	(struct sf_private_tag*, const double *ptr, sf_count_t len) ;
 
 	sf_count_t		(*seek) 		(struct sf_private_tag*, int mode, sf_count_t samples_from_start) ;
 	int				(*write_header)	(struct sf_private_tag*, int calc_length) ;
@@ -238,14 +317,21 @@ typedef struct sf_private_tag
 	int				(*close)		(struct sf_private_tag*) ;
 
 	char			*format_desc ;
+
+	/* Virtual I/O functions. */
+	int					virtual_io ;
+	SF_VIRTUAL_IO		vio ;
+	void				*vio_user_data ;
 } SF_PRIVATE ;
 
 
 
 enum
-{	SFE_NO_ERROR		= SF_ERR_NO_ERROR,
-	SFE_BAD_OPEN_FORMAT	= SF_ERR_UNRECOGNISED_FORMAT,
-	SFE_SYSTEM			= SF_ERR_SYSTEM,
+{	SFE_NO_ERROR				= SF_ERR_NO_ERROR,
+	SFE_BAD_OPEN_FORMAT			= SF_ERR_UNRECOGNISED_FORMAT,
+	SFE_SYSTEM					= SF_ERR_SYSTEM,
+	SFE_MALFORMED_FILE			= SF_ERR_MALFORMED_FILE,
+	SFE_UNSUPPORTED_ENCODING	= SF_ERR_UNSUPPORTED_ENCODING,
 
 	SFE_BAD_FILE,
 	SFE_BAD_FILE_READ,
@@ -271,12 +357,13 @@ enum
 	SFE_NO_PIPE_WRITE,
 
 	SFE_INTERNAL,
-	SFE_LOG_OVERRUN,
 	SFE_BAD_CONTROL_CMD,
 	SFE_BAD_ENDIAN,
 	SFE_CHANNEL_COUNT,
 	SFE_BAD_RDWR_FORMAT,
 
+	SFE_BAD_VIRTUAL_IO,
+
 	SFE_INTERLEAVE_MODE,
 	SFE_INTERLEAVE_SEEK,
 	SFE_INTERLEAVE_READ,
@@ -290,27 +377,28 @@ enum
 	SFE_BAD_OPEN_MODE,
 	SFE_OPEN_PIPE_RDWR,
 	SFE_RDWR_POSITION,
+	SFE_RDWR_BAD_HEADER,
 
 	SFE_STR_NO_SUPPORT,
+	SFE_STR_NOT_WRITE,
 	SFE_STR_MAX_DATA,
 	SFE_STR_MAX_COUNT,
 	SFE_STR_BAD_TYPE,
 	SFE_STR_NO_ADD_END,
 	SFE_STR_BAD_STRING,
 	SFE_STR_WEIRD,
-	SFE_RDWR_BAD_HEADER,
 
 	SFE_WAV_NO_RIFF,
 	SFE_WAV_NO_WAVE,
 	SFE_WAV_NO_FMT,
 	SFE_WAV_FMT_SHORT,
-	SFE_WAV_FMT_TOO_BIG,
 	SFE_WAV_BAD_FACT,
 	SFE_WAV_BAD_PEAK,
 	SFE_WAV_PEAK_B4_FMT,
 	SFE_WAV_BAD_FORMAT,
 	SFE_WAV_BAD_BLOCKALIGN,
 	SFE_WAV_NO_DATA,
+	SFE_WAV_BAD_LIST,
 	SFE_WAV_ADPCM_NOT4BIT,
 	SFE_WAV_ADPCM_CHANNELS,
 	SFE_WAV_GSM610_FORMAT,
@@ -404,6 +492,21 @@ enum
 	SFE_SDS_NOT_SDS,
 	SFE_SDS_BAD_BIT_WIDTH,
 
+	SFE_SD2_FD_DISALLOWED,
+	SFE_SD2_BAD_DATA_OFFSET,
+	SFE_SD2_BAD_MAP_OFFSET,
+	SFE_SD2_BAD_DATA_LENGTH,
+	SFE_SD2_BAD_MAP_LENGTH,
+	SFE_SD2_BAD_RSRC,
+	SFE_SD2_BAD_SAMPLE_SIZE,
+
+	SFE_FLAC_BAD_HEADER,
+	SFE_FLAC_NEW_DECODER,
+	SFE_FLAC_INIT_DECODER,
+	SFE_FLAC_LOST_SYNC,
+	SFE_FLAC_BAD_SAMPLE_RATE,
+	SFE_FLAC_UNKOWN_ERROR,
+
 	SFE_MAX_ERROR			/* This must be last in list. */
 } ;
 
@@ -467,6 +570,7 @@ int		psf_calc_max_all_channels	(SF_PRIVATE *psf, double *peaks, int normalize) ;
 /* Functions in strings.c. */
 
 const char* psf_get_string (SF_PRIVATE *psf, int str_type) ;
+int psf_set_string (SF_PRIVATE *psf, int str_type, const char *str) ;
 int psf_store_string (SF_PRIVATE *psf, int str_type, const char *str) ;
 
 /* Default seek function. Use for PCM and float encoded data. */
@@ -484,21 +588,29 @@ int macos_guess_file_type (SF_PRIVATE *psf, const char *filename) ;
 
 int psf_fopen (SF_PRIVATE *psf, const char *pathname, int flags) ;
 int psf_set_stdio (SF_PRIVATE *psf, int mode) ;
-int psf_filedes_valid (SF_PRIVATE *psf) ;
+int psf_file_valid (SF_PRIVATE *psf) ;
 void psf_set_file (SF_PRIVATE *psf, int fd) ;
+void psf_init_files (SF_PRIVATE *psf) ;
+void psf_use_rsrc (SF_PRIVATE *psf, int on_off) ;
 
 sf_count_t psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence) ;
 sf_count_t psf_fread (void *ptr, sf_count_t bytes, sf_count_t count, SF_PRIVATE *psf) ;
-sf_count_t psf_fwrite (void *ptr, sf_count_t bytes, sf_count_t count, SF_PRIVATE *psf) ;
+sf_count_t psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t count, SF_PRIVATE *psf) ;
 sf_count_t psf_fgets (char *buffer, sf_count_t bufsize, SF_PRIVATE *psf) ;
 sf_count_t psf_ftell (SF_PRIVATE *psf) ;
 sf_count_t psf_get_filelen (SF_PRIVATE *psf) ;
 
+void psf_fsync (SF_PRIVATE *psf) ;
+
 int psf_is_pipe (SF_PRIVATE *psf) ;
 
 int psf_ftruncate (SF_PRIVATE *psf, sf_count_t len) ;
 int psf_fclose (SF_PRIVATE *psf) ;
 
+/* Open and close the resource fork of a file. */
+int psf_open_rsrc (SF_PRIVATE *psf, int mode) ;
+int psf_close_rsrc (SF_PRIVATE *psf) ;
+
 /*
 void psf_fclearerr (SF_PRIVATE *psf) ;
 int psf_ferror (SF_PRIVATE *psf) ;
@@ -520,18 +632,20 @@ int		nist_open	(SF_PRIVATE *psf) ;
 int		paf_open	(SF_PRIVATE *psf) ;
 int		pvf_open	(SF_PRIVATE *psf) ;
 int		raw_open	(SF_PRIVATE *psf) ;
+int		sd2_open	(SF_PRIVATE *psf) ;
 int		sds_open	(SF_PRIVATE *psf) ;
 int		svx_open	(SF_PRIVATE *psf) ;
 int		voc_open	(SF_PRIVATE *psf) ;
 int		w64_open	(SF_PRIVATE *psf) ;
 int		wav_open	(SF_PRIVATE *psf) ;
 int		xi_open		(SF_PRIVATE *psf) ;
+int		flac_open	(SF_PRIVATE *psf) ;
+int		caf_open	(SF_PRIVATE *psf) ;
 
 /* In progress. Do not currently work. */
 
 int		ogg_open	(SF_PRIVATE *psf) ;
 int		rx2_open	(SF_PRIVATE *psf) ;
-int		sd2_open	(SF_PRIVATE *psf) ;
 int		txw_open	(SF_PRIVATE *psf) ;
 int		wve_open	(SF_PRIVATE *psf) ;
 int		dwd_open	(SF_PRIVATE *psf) ;
@@ -542,14 +656,15 @@ int		macbinary3_open (SF_PRIVATE *psf) ;
 **	Init functions for a number of common data encodings.
 */
 
-int 	pcm_init		(SF_PRIVATE *psf) ;
-int 	ulaw_init		(SF_PRIVATE *psf) ;
-int 	alaw_init		(SF_PRIVATE *psf) ;
-int 	float32_init	(SF_PRIVATE *psf) ;
-int 	double64_init	(SF_PRIVATE *psf) ;
-int 	dwvw_init		(SF_PRIVATE *psf, int bitwidth) ;
-int		gsm610_init 	(SF_PRIVATE *psf) ;
-int		vox_adpcm_init 	(SF_PRIVATE *psf) ;
+int		pcm_init		(SF_PRIVATE *psf) ;
+int		ulaw_init		(SF_PRIVATE *psf) ;
+int		alaw_init		(SF_PRIVATE *psf) ;
+int		float32_init	(SF_PRIVATE *psf) ;
+int		double64_init	(SF_PRIVATE *psf) ;
+int		dwvw_init		(SF_PRIVATE *psf, int bitwidth) ;
+int		gsm610_init		(SF_PRIVATE *psf) ;
+int		vox_adpcm_init	(SF_PRIVATE *psf) ;
+int		flac_init		(SF_PRIVATE *psf) ;
 
 int 	dither_init		(SF_PRIVATE *psf, int mode) ;
 
@@ -571,7 +686,7 @@ void	*psf_memset (void *s, int c, sf_count_t n) ;
 ** Systems without these functions should use the
 */
 
-#if (defined (WIN32) || defined (_WIN32))
+#if USE_WINDOWS_API
 #define	LSF_SNPRINTF	_snprintf
 #elif		(HAVE_SNPRINTF && ! FORCE_MISSING_SNPRINTF)
 #define	LSF_SNPRINTF	snprintf
@@ -580,7 +695,7 @@ int missing_snprintf (char *str, size_t n, char const *fmt, ...) ;
 #define	LSF_SNPRINTF	missing_snprintf
 #endif
 
-#if (defined (WIN32) || defined (_WIN32))
+#if USE_WINDOWS_API
 #define	LSF_VSNPRINTF	_vsnprintf
 #elif		(HAVE_VSNPRINTF && ! FORCE_MISSING_SNPRINTF)
 #define	LSF_VSNPRINTF	vsnprintf
@@ -589,7 +704,7 @@ int missing_vsnprintf (char *str, size_t n, const char *fmt, ...) ;
 #define	LSF_VSNPRINTF	missing_vsnprintf
 #endif
 
-#endif /* COMMON_H_INCLUDED */
+#endif /* SNDFILE_COMMON_H */
 
 /*------------------------------------------------------------------------------------
 ** Extra commands for sf_command(). Not for public use yet.
