@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002,2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,7 @@
 #include	<ctype.h>
 
 #include	"sndfile.h"
-#include	"config.h"
+#include	"sfconfig.h"
 #include	"sfendian.h"
 #include	"common.h"
 #include	"float_cast.h"
@@ -153,7 +153,8 @@ mat5_close	(SF_PRIVATE *psf)
 
 static int
 mat5_write_header (SF_PRIVATE *psf, int calc_length)
-{	static const char	*sr_name = "samplerate\0\0\0\0\0\0\0\0\0\0\0" ;
+{	static const char	*filename = "MATLAB 5.0 MAT-file, written by " PACKAGE "-" VERSION ", " ;
+	static const char	*sr_name = "samplerate\0\0\0\0\0\0\0\0\0\0\0" ;
 	static const char	*wd_name = "wavedata\0" ;
 	sf_count_t	current, datasize ;
 	int			encoding ;
@@ -202,12 +203,11 @@ mat5_write_header (SF_PRIVATE *psf, int calc_length)
 	psf->headindex = 0 ;
 	psf_fseek (psf, 0, SEEK_SET) ;
 
-	psf_binheader_writef (psf, "S", "MATLAB 5.0 MAT-file, written by " PACKAGE "-" VERSION ", ") ;
-	psf_get_date_str ((char*) psf->buffer, sizeof (psf->buffer)) ;
-	psf_binheader_writef (psf, "jS", -1, psf->buffer) ;
+	psf_get_date_str (psf->u.cbuf, sizeof (psf->u.scbuf)) ;
+	psf_binheader_writef (psf, "bb", filename, strlen (filename), psf->u.cbuf, strlen (psf->u.cbuf) + 1) ;
 
-	memset (psf->buffer, ' ', 124 - psf->headindex) ;
-	psf_binheader_writef (psf, "b", psf->buffer, 124 - psf->headindex) ;
+	memset (psf->u.scbuf, ' ', 124 - psf->headindex) ;
+	psf_binheader_writef (psf, "b", psf->u.scbuf, 124 - psf->headindex) ;
 
 	psf->rwf_endian = psf->endian ;
 
@@ -260,15 +260,15 @@ mat5_read_header (SF_PRIVATE *psf)
 	short	version, endian ;
 	int		type, size, flags1, flags2, rows, cols ;
 
-	psf_binheader_readf (psf, "pb", 0, psf->buffer, 124) ;
+	psf_binheader_readf (psf, "pb", 0, psf->u.cbuf, 124) ;
 
-	psf->buffer [125] = 0 ;
+	psf->u.scbuf [125] = 0 ;
 
-	if (strlen ((char*) psf->buffer) >= 124)
+	if (strlen (psf->u.cbuf) >= 124)
 		return SFE_UNIMPLEMENTED ;
 
-	if (strstr ((char*) psf->buffer, "MATLAB 5.0 MAT-file") == (char*) psf->buffer)
-		psf_log_printf (psf, "%s\n", psf->buffer) ;
+	if (strstr (psf->u.cbuf, "MATLAB 5.0 MAT-file") == psf->u.cbuf)
+		psf_log_printf (psf, "%s\n", psf->u.scbuf) ;
 
 
 	psf_binheader_readf (psf, "E22", &version, &endian) ;
